import os
import uuid
from typing import Optional

FOLDER_NAME = ".awesome_folder"


class ModifyFile:
    def __init__(self, folder_name: str = FOLDER_NAME):
        self.folder_name = folder_name
        self._mkdir()

    def modify(self, namespace: str = "default", content: Optional[str] = None):
        path = self._mkdir(namespace)
        filename = str(uuid.uuid4())
        content = content or filename

        self._write(path, filename, content)

    def _mkdir(self, name: str = "") -> str:
        path = os.path.join(self.folder_name, name)
        if not os.path.exists(path):
            os.mkdir(path)

        return path

    def _write(self, path: str, filename: str, content: str):
        f = open(os.path.join(path, filename), "w+")
        f.write(content)
        f.close()



from awesome_git_mosaic.adapters.console.console_adapter import ConsoleAdapter


class WriteInConsole:
    def __init__(self, console_adapter: ConsoleAdapter = None):
        self.console_adapter = console_adapter or ConsoleAdapter()

    def write(
        self,
        text: str,
        with_spaces: bool = True,
        background: bool = False,
        inverted: bool = False,
    ):
        output = self.console_adapter.output(text, with_spaces, background, inverted)
        print(output)



from typing import TYPE_CHECKING, Optional

from awesome_git_mosaic.charmaps.basic.basic_charmap import BasicCharmap

if TYPE_CHECKING:
    from awesome_git_mosaic.charmaps.charmap import Charmap


CONSOLE_PIXEL = "▉"
CONSOLE_SPACE = " "
CONSOLE_BG = "░"


class ConsoleAdapter:
    def __init__(self, charmap: Optional["Charmap"] = None):
        self.charmap = charmap or BasicCharmap()

    def output(
        self,
        message: str,
        with_spaces: bool = True,
        background: bool = False,
        inverted: bool = False,
    ) -> str:
        lines = self.charmap.translate(message, with_spaces, inverted)

        bg = CONSOLE_BG if background else CONSOLE_SPACE
        output = []
        for line in lines:
            output_line = ""
            for char in line:
                output_line += CONSOLE_PIXEL if self.charmap.is_pixel(char) else bg
            output.append(output_line)

        return "\n".join(output)



from datetime import datetime

from awesome_git_mosaic.adapters.git_mosaic.git_mosaic_adapter import GitMosaicAdapter
from awesome_git_mosaic.gateways.git.git_gateway import GitGateway
from awesome_git_mosaic.usecases.modify_file import ModifyFile


class WriteMosaic:
    def __init__(
        self,
        git_mosaic_adapter: GitMosaicAdapter = None,
        git_gateway: GitGateway = None,
        modify_file: ModifyFile = None,
    ):
        self.git_mosaic_adapter = git_mosaic_adapter or GitMosaicAdapter()
        self.git_gateway = git_gateway or GitGateway()
        self.modify_file = modify_file or ModifyFile()

    def write(
        self,
        message: str,
        strength: int = 15,
        multiply: int = 1,
        with_spaces: bool = True,
        background: bool = False,
        inverted: bool = False,
    ):
        timestamps: list[datetime] = []
        if background:
            bgstr = " " * (len(message) * multiply)
            timestamps += self.git_mosaic_adapter.output(
                bgstr, datetime.today(), with_spaces, True
            )

        for _ in range(strength):
            timestamps += self.git_mosaic_adapter.output(
                f"{message} " * multiply, None, with_spaces, inverted
            )

        timestamps.sort()

        total = len(timestamps)
        count = 0.0
        last_pct = 0

        print(f"Disabling git Garbage Collector...")
        self.git_gateway.disable_garbage_collector()

        print(f"Creating {total} commits...")
        for timestamp in timestamps:
            self.modify_file.modify(timestamp.ctime())
            self.git_gateway.add()
            self.git_gateway.commit(f"{timestamp.ctime()} {count}", timestamp)

            count += 1
            pct = round(count * 100 / total)

            if pct % 5 == 0 and pct != last_pct:
                print(f"{pct}%")
                last_pct = pct

            if count % 500 == 0:
                print(f"{count} commits created, pushing")
                self.git_gateway.push()

        print("All commits created. Last push...")
        self.git_gateway.push()

        print(f"Re-enabling git Garbage Collector...")
        self.git_gateway.enable_garbage_collector()

        print("That's all, folks! Bye!")
